@@ -109,56 +109,7 @@ fn _start_chr_128(buf: &[u8], c: u8) -> Option<usize> {
         }
     }
     // the remaining data is the max: 15 bytes.
-    _memrchr_remaining_15_bytes_impl(buf_ptr_cur, c, start_ptr)
-}
-
-#[inline(always)]
-pub(crate) fn _memrchr_remaining_15_bytes_impl(
-    buf_ptr_cur: *const u8,
-    c: u8,
-    start_ptr: *const u8,
-) -> Option<usize> {
-    let mut buf_ptr_cur = buf_ptr_cur;
-    let cc: u64 = _c8_value(c);
-    {
-        let loop_size = 8;
-        let buf_ptr = unsafe { buf_ptr_cur.sub(loop_size) };
-        if buf_ptr >= start_ptr {
-            _unroll_one_chr_8!(buf_ptr, cc, start_ptr, loop_size, 0);
-            //
-            buf_ptr_cur = buf_ptr;
-        }
-    }
-    {
-        let loop_size = 4;
-        let cc: u32 = cc as u32;
-        let buf_ptr = unsafe { buf_ptr_cur.sub(loop_size) };
-        if buf_ptr >= start_ptr {
-            _unroll_one_chr_4!(buf_ptr, cc, start_ptr, loop_size, 0);
-            //
-            buf_ptr_cur = buf_ptr;
-        }
-    }
-    {
-        let loop_size = 2;
-        let cc: u16 = cc as u16;
-        let buf_ptr = unsafe { buf_ptr_cur.sub(loop_size) };
-        if buf_ptr >= start_ptr {
-            _unroll_one_chr_2!(buf_ptr, cc, start_ptr, loop_size, 0);
-            //
-            buf_ptr_cur = buf_ptr;
-        }
-    }
-    {
-        let loop_size = 1;
-        let cc: u8 = c;
-        let buf_ptr = unsafe { buf_ptr_cur.sub(loop_size) };
-        if buf_ptr >= start_ptr {
-            _unroll_one_chr_1!(buf_ptr, cc, start_ptr, loop_size, 0);
-        }
-    }
-    //
-    None
+    _memrchr_remaining_15_bytes_impl(buf_ptr_cur, cc as u64, start_ptr)
 }
 
 #[cfg(target_pointer_width = "64")]
@@ -202,47 +153,8 @@ fn _start_chr_64(buf: &[u8], c: u8) -> Option<usize> {
             buf_ptr_cur = buf_ptr;
         }
     }
-    // a rest data is a max: 7 bytes.
-    _memrchr_remaining_7_bytes_impl(buf_ptr_cur, c, start_ptr)
-}
-
-#[inline(always)]
-pub(crate) fn _memrchr_remaining_7_bytes_impl(
-    buf_ptr_cur: *const u8,
-    c: u8,
-    start_ptr: *const u8,
-) -> Option<usize> {
-    let mut buf_ptr_cur = buf_ptr_cur;
-    let cc: u32 = _c4_value(c);
-    {
-        let loop_size = 4;
-        let buf_ptr = unsafe { buf_ptr_cur.sub(loop_size) };
-        if buf_ptr >= start_ptr {
-            _unroll_one_chr_4!(buf_ptr, cc, start_ptr, loop_size, 0);
-            //
-            buf_ptr_cur = buf_ptr;
-        }
-    }
-    {
-        let loop_size = 2;
-        let cc: u16 = cc as u16;
-        let buf_ptr = unsafe { buf_ptr_cur.sub(loop_size) };
-        if buf_ptr >= start_ptr {
-            _unroll_one_chr_2!(buf_ptr, cc, start_ptr, loop_size, 0);
-            //
-            buf_ptr_cur = buf_ptr;
-        }
-    }
-    {
-        let loop_size = 1;
-        let cc: u8 = c;
-        let buf_ptr = unsafe { buf_ptr_cur.sub(loop_size) };
-        if buf_ptr >= start_ptr {
-            _unroll_one_chr_1!(buf_ptr, cc, start_ptr, loop_size, 0);
-        }
-    }
-    //
-    None
+    // the remaining data is the max: 7 bytes.
+    _memrchr_remaining_7_bytes_impl(buf_ptr_cur, cc as u32, start_ptr)
 }
 
 #[cfg(target_pointer_width = "32")]
@@ -286,18 +198,57 @@ fn _start_chr_32(buf: &[u8], c: u8) -> Option<usize> {
             buf_ptr_cur = unsafe { buf_ptr.add(loop_size) };
         }
     }
-    // a rest data is a max: 3 bytes.
-    _memrchr_remaining_3_bytes_impl(buf_ptr_cur, c, start_ptr)
+    // the remaining data is the max: 3 bytes.
+    _memrchr_remaining_3_bytes_impl(buf_ptr_cur, cc as u16, start_ptr)
+}
+
+#[inline(always)]
+pub(crate) fn _memrchr_remaining_15_bytes_impl(
+    buf_ptr_cur: *const u8,
+    cc: u64,
+    start_ptr: *const u8,
+) -> Option<usize> {
+    let mut buf_ptr_cur = buf_ptr_cur;
+    {
+        let loop_size = 8;
+        let buf_ptr = unsafe { buf_ptr_cur.sub(loop_size) };
+        if buf_ptr >= start_ptr {
+            _unroll_one_chr_8!(buf_ptr, cc, start_ptr, loop_size, 0);
+            //
+            buf_ptr_cur = buf_ptr;
+        }
+    }
+    // the remaining data is the max: 7 bytes.
+    _memrchr_remaining_7_bytes_impl(buf_ptr_cur,  cc as u32, start_ptr)
+}
+
+#[inline(always)]
+pub(crate) fn _memrchr_remaining_7_bytes_impl(
+    buf_ptr_cur: *const u8,
+    cc: u32,
+    start_ptr: *const u8,
+) -> Option<usize> {
+    let mut buf_ptr_cur = buf_ptr_cur;
+    {
+        let loop_size = 4;
+        let buf_ptr = unsafe { buf_ptr_cur.sub(loop_size) };
+        if buf_ptr >= start_ptr {
+            _unroll_one_chr_4!(buf_ptr, cc, start_ptr, loop_size, 0);
+            //
+            buf_ptr_cur = buf_ptr;
+        }
+    }
+    // the remaining data is the max: 3 bytes.
+    _memrchr_remaining_3_bytes_impl(buf_ptr_cur, cc as u16, start_ptr)
 }
 
 #[inline(always)]
 pub(crate) fn _memrchr_remaining_3_bytes_impl(
     buf_ptr_cur: *const u8,
-    c: u8,
+    cc: u16,
     start_ptr: *const u8,
 ) -> Option<usize> {
     let mut buf_ptr_cur = buf_ptr_cur;
-    let cc: u16 = _c2_value(c);
     {
         let loop_size = 2;
         let buf_ptr = unsafe { buf_ptr_cur.sub(loop_size) };
@@ -309,7 +260,7 @@ pub(crate) fn _memrchr_remaining_3_bytes_impl(
     }
     {
         let loop_size = 1;
-        let cc: u8 = c;
+        let cc: u8 = cc as u8;
         let buf_ptr = unsafe { buf_ptr_cur.sub(loop_size) };
         if buf_ptr >= start_ptr {
             _unroll_one_chr_1!(buf_ptr, cc, start_ptr, loop_size, 0);
@@ -360,7 +311,7 @@ fn _start_chr_16(buf: &[u8], c: u8) -> Option<usize> {
             buf_ptr_cur = unsafe { buf_ptr.add(loop_size) };
         }
     }
-    // a rest data is a max: 1 bytes.
+    // the remaining data is the max: 1 bytes.
     {
         let loop_size = 1;
         let cc: u8 = c;
